package com.example.ueberwachungssystem;

import static java.lang.Math.sqrt;

import android.content.Context;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.widget.TextView;

import java.util.concurrent.LinkedBlockingQueue;


/**
 * Accelerometer inherits some methods from abstract Detector class (more info there)
 *
 *
 * USE FROM MAIN ACTIVITY:
 *
 * Accelerometer beschleunigungssensor = new Accelerometer(this);
 * onCreate:
 * //Accelerometer Setup
 * beschleunigungssensor = new Accelerometer(this, logger, textViewLog);   //logger and textview only for debugging necessary
 * beschleunigungssensor.getSensor();
 *
 * //Starting Detection:
 * beschleunigungssensor.startDetection();
 * //Stopping Detection: also recommended at onPause to avoid unnecessary battery consumption
 * beschleunigungssensor.stopDetection();
 *
 * */

public class Accelerometer extends Detector implements SensorEventListener {

    Logger logger;
    public SensorManager sensorManager;
    private static final int sensorType = Sensor.TYPE_LINEAR_ACCELERATION;
    private Sensor accelerometer;
    private Context context;
    boolean alarm = false;
    //Preallocate memory for the data of each axis of the acceleration sensor
    float x;
    float y;
    float z;
    float betrag; //Betrag aller drei Achsen sqrt(x*x + y*y + z*z)
    private DetectionReport detectionReport;
    TextView textView;

    //Queue for taking in values
    LinkedBlockingQueue<Float> linkedBlockingQueue = new LinkedBlockingQueue<>();


    // In constructor pass Activity, Context and TextView from MainActivity in Accelerometer class
    public Accelerometer(Context context, Logger mainLogger, TextView textView){
        super(context); //von Detektor
        logger = mainLogger;
        this.context = context;
        this.textView = this.textView;
        //Logger logger = new Logger(this.getClass().getSimpleName(), textViewLog, "");
    }

    public void getSensor(){
        sensorManager = (SensorManager)context.getSystemService(Context.SENSOR_SERVICE);
        if(sensorManager.getSensorList(sensorType).size()==0) {
            logger.log("Es gibt den gewünschten Sensor nicht");
            accelerometer = null;
        }
        else {
            accelerometer = sensorManager.getSensorList(sensorType).get(0);
            logger.log("Sensor gefunden");
        }
    }

    @Override
    public void onSensorChanged(SensorEvent event) {
        try {
            checkAlarm(event);
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        }
    }

    StringBuffer stringBuffer = new StringBuffer();

    public void checkAlarm (SensorEvent event) throws InterruptedException {
        x = event.values[0];
        y = event.values[1];
        z = event.values[2];
        betrag = (float) sqrt(x*x + y*y + z*z);
        float threshold = 1.5F;

        if (!alarm) {
            if (betrag > threshold) {
                alarm = true;
                detectionReport = new DetectionReport("Accelerometer1", "Bewegung", betrag);

                stringBuffer.append("\nDetectionReport = ").append(detectionReport).append("\nAlarm an");
                logger.clearLog();
                logger.log(stringBuffer.toString());
                stringBuffer.delete(0, stringBuffer.length());
            }
        } else {
            if (betrag < threshold) {
                stringBuffer.append("\nAlarm noch an; Neuer Betragswert unter Schwellwert:").append(betrag);
                //logger.log("Alarm ist noch an; Neuer Betragswert unter Schwellwert: " + betrag);
                alarm = false;
                //logger.log("Alarm" + alarm);
                //logger.log("Alarm wieder ausgeschaltet");
                stringBuffer.append("\nAlarm = ").append(alarm);
            } else {
                logger.log("Betragswert immer noch über Schwellwert: " + betrag + "; Alarm bleibt an.");
            }
            //logger.clearLog();
            logger.log(stringBuffer.toString());
            stringBuffer.delete(0, stringBuffer.length());
        }
    }

    @Override
    public void onAccuracyChanged(Sensor sensor, int accuracy) {
    }

    @Override
    public void startDetection() {
        // entspricht void start()
        if (accelerometer != null) {
            sensorManager.registerListener(this, accelerometer, SensorManager.SENSOR_DELAY_GAME);
            logger.log("Sensor registriert");
        }
    }

    @Override
    public void stopDetection() {
        // entspricht void stop()
        sensorManager.unregisterListener(this, accelerometer);
        logger.log("Vom Sensor abgemeldet");
    }
}